const webpack = require('webpack');

// const { APP_ID, ENDPOINT_AUTH, CDN } = process.env;

const internalNodeModulesRegExp = /@specialcircumstances(?!.*node_modules)/;
const externalNodeModulesRegExp = /node_modules(?!\/@specialcircumstances(?!.*node_modules))/;

module.exports = {
  webpack: (config, { defaultLoaders }) => {
    // Allows external scripts to be transpiled with the App itself
    // Set false to use `npm link` or `lerna bootstrap`
    config.resolve.symlinks = false;
    config.externals = config.externals.map(external => {
      if (typeof external !== 'function') {
        return external;
      }
      return (ctx, req, cb) =>
        internalNodeModulesRegExp.test(req) ? cb() : external(ctx, req, cb);
    });

    // Loaders
    config.module.rules.push({
      test: /\.+(js|jsx)$/,
      loader: defaultLoaders.babel,
      include: [internalNodeModulesRegExp]
    });

    // Plugins
    config.plugins.push(
      // Provides env variables to the client.
      // CAUTION: Don't expose any app secrets or private keys.
      new webpack.DefinePlugin({
        'process.env': {
          // APP_ID: JSON.stringify(APP_ID),
          // CDN: JSON.stringify(CDN),
          // ENDPOINT_AUTH: JSON.stringify(ENDPOINT_AUTH)
        }
      })
    );

    return config;
  },
  webpackDevMiddleware: config => {
    const ignored = [config.watchOptions.ignored[0], externalNodeModulesRegExp];
    config.watchOptions.ignored = ignored;
    return config;
  }
};
