import { Component } from 'react';
import { Link } from '../../src/server/routes';
import withRedux from 'next-redux-wrapper';
import withLayout from '../../hoc/layout';

// Store
import initStore from '../../src/store';

// Actions
import { fetchAllStories, storiesSelector } from '../../src/reducers';

// Components
import {
  MDCDisplayText,
  MDCHeadline,
  MDCTitle,
  MDCBodyText
} from '@specialcircumstances/pkg-mdc';
import Main from '../../src/components/main';

import { breakpoints } from '../../src/config/styles';
const { tablet, desktop, phone } = breakpoints;

// API_ENDPOINT
const apiEndpoint = 'http://localhost:9000';

// Map State to Page Properties
const mapStateToProps = state => {
  return {
    stories: storiesSelector(state),
    images: state.getIn(['entities', 'images'])
  };
};

class StoriesPage extends Component {
  static async getInitialProps({ store: { dispatch } }) {
    await dispatch(fetchAllStories(apiEndpoint));
  }

  render() {
    const { stories = [], images = {} } = this.props;

    return (
      <Main>
        <div className="stories__title">
          <MDCDisplayText size="1">Stories</MDCDisplayText>
        </div>

        <div className="card-list">
          {stories.map(a => {
            const imageID = a.images ? a.images[0] : null;
            const image = images[imageID] || {};
            const url = `/assets/${image.id}/300/${image.name}`;

            return (
              <Link key={a._id} route="story" params={{ id: a._id }}>
                <a className="mdc-card card__title--action">
                  <div
                    className="mdc-card__media mdc-card__media--16-9"
                    style={{ backgroundImage: `url(${url})` }}
                  />
                  <div className="card__title">
                    <MDCTitle>{a.title}</MDCTitle>
                    <MDCBodyText>Cars</MDCBodyText>
                  </div>
                </a>
              </Link>
            );
          })}
        </div>

        <style global jsx>{`
          @import '@material/card/dist/mdc.card.css';
        `}</style>
        <style jsx>{`
          .stories__title {
            padding-top: 8px;
            margin-left: 2%;
            margin-right: 2%;
          }
          .card-list {
            display: flex;
            flex-wrap: wrap;
            margin: 1%;
          }
          .mdc-card__media {
            background-position-y: bottom;
          }
          .card__title {
            margin: 0 8px;
            overflow: hidden;
            text-overflow: ellipsis;
          }
          .card__title--action {
            text-decoration: none;
            font-size: 1.1rem;
            color: var(--mdc-theme-primary);
            white-space: nowrap;
          }
          .mdc-card {
            width: 48%;
            margin: 1%;
          }
          @media only screen and (max-width: ${phone}px) {
            .mdc-card {
              width: 98%;
              margin: 1%;
            }
          }
          @media only screen and (min-width: ${tablet}px) {
            .mdc-card {
              width: 31.3333%;
              margin: 1%;
            }
          }
          @media only screen and (min-width: ${desktop}px) {
            .mdc-card {
              width: 24%;
              margin: 0.5%;
            }
          }
        `}</style>
      </Main>
    );
  }
}

export default withRedux(initStore, mapStateToProps)(withLayout(StoriesPage));
