import { Component } from 'react';
// import { Value } from 'slate';

import withRedux from 'next-redux-wrapper';
import withLayout from '../../hoc/layout';

// Store
import initStore from '../../src/store';

// Actions
import { setEntities } from '@specialcircumstances/pkg-entities';
import { setUI, updateUI } from '@specialcircumstances/pkg-ui';
import { setEditor } from '@specialcircumstances/pkg-editor';
import { fetchStory, storySelector, updateStory } from '../../src/reducers';

// Local Components
import Editor from '../../src/editor';
import Main from '../../src/components/main';

import { layout } from '../../src/config/styles';

const { max } = layout;

// API_ENDPOINT
const apiEndpoint = 'http://localhost:9000';

// Map State to Page Properties
const mapStateToProps = state => {
  return {
    story: storySelector(state),
    menu: state.getIn(['ui', 'menu']),
    toolbar: state.getIn(['ui', 'toolbar']),
    editor: state.getIn(['editor', 'value'])
  };
};

// Map Dispatch to Page Properties
const mapDispatchToProps = dispatch => ({
  handleDirtyFlag: value => dispatch(setEditor('dirty', value)),
  handleUpdateStory: (id, value) => {
    dispatch(setEntities('stories', id, 'content', value));
  },
  handleSaveStory: (id, content) => {
    dispatch(updateStory(apiEndpoint, id, content));
  },
  handleFetchStory: id => dispatch(fetchStory('', id)),
  handleSetMenu: (name, value) => dispatch(setUI('menu', name, value)),
  handleUpdateMenu: data => dispatch(updateUI('menu', data))
});

class StoryPage extends Component {
  static async getInitialProps({ query: { id } }) {
    return { id };
  }

  componentDidMount() {
    const { id, handleFetchStory } = this.props;
    handleFetchStory(id);
  }

  render() {
    const {
      menu,
      story,
      toolbar,
      isDirty,
      handleDirtyFlag,
      handleSetMenu,
      handleSaveStory,
      handleUpdateMenu,
      handleUpdateStory
    } = this.props;

    // loading state
    if (!story || !story.content || !story.content.change) {
      return null;
    }
    const { id, content } = story;

    return (
      <Main>
        <div className="evt-editor">
          <Editor
            id={id}
            menu={menu}
            toolbar={toolbar}
            value={content}
            isDirty={isDirty}
            handleDirtyFlag={handleDirtyFlag}
            handleSetMenu={handleSetMenu}
            handleSaveStory={handleSaveStory}
            handleUpdateMenu={handleUpdateMenu}
            handleUpdateStory={handleUpdateStory}
          />
        </div>

        <style jsx>{`
          .evt-editor {
            max-width: ${max}px;
            margin: 0 auto;
            padding: 0 32px;
          }
        `}</style>
      </Main>
    );
  }
}

export default withRedux(initStore, mapStateToProps, mapDispatchToProps)(
  withLayout(StoryPage)
);
