import { Component, Fragment } from 'react';
import debounce from 'lodash.debounce';
// import throttle from 'lodash.throttle';
import { Editor } from 'slate-react';

// Editor Components
import marks from './marks';
import nodes from './nodes';
import schema from './schema';
import renderPlaceholder from './placeholder';
import { insertImage, insertGallery } from './insert';

// React Components
import Menu from './menu';
import Toolbar from './toolbar';
import Timestamp from '../components/timestamp';
import Divider from '../components/divider';

// Temp Data
// import { demoimage } from '../config';

const EDITOR_TIMEOUT = 3000;

class StoryEditor extends Component {
  componentDidMount() {
    // const { isDirty } = this.props;
    // window.addEventListener('beforeunload', this.preventNavigation);
  }

  componentWillUnMount() {
    // const { isDirty } = this.props;
    // window.removeEventListener('beforeunload', this.preventNavigation);
  }

  preventNavigation = e => {
    const { isDirty } = this.props;
    if (isDirty) {
      e.returnValue =
        'Saving is in progress, please wait a moment and try again.';
    }
  };
  // On change, update the app's React state with the new editor value and
  // send the changes to the Redux store
  onChange = ({ value }) => {
    const { id, handleUpdateStory, handleDirtyFlag } = this.props;

    this.saveStory(value);
    handleDirtyFlag(true);
    handleUpdateStory(id, value);
  };

  // move to reducer...
  saveStory = debounce(
    value => {
      const { id, handleSaveStory } = this.props;
      handleSaveStory(id, value);
    },
    EDITOR_TIMEOUT,
    { trailing: true, leading: true }
  );

  insertGallery = () => {
    const change = this.props.value.change().call(insertGallery);
    this.onChange(change);
  };

  insertImage = options => {
    const change = this.props.value.change().call(insertImage, options);
    this.onChange(change);
  };

  componentDidUpdate = () => {
    const { menu, value, handleSetMenu } = this.props;
    if (!value) {
      return;
    }
    const { anchorBlock, isBlurred } = value;
    const { key, isEmpty, type } = anchorBlock;

    const block = document.querySelector(`[data-key="${key}"]`);
    const top = block && block.offsetTop + 2;
    const left = block && block.offsetLeft;
    const visible = isEmpty && type === 'paragraph';
    const open = isBlurred && menu.open;

    handleSetMenu('top', top);
    handleSetMenu('left', left);
    handleSetMenu('visible', visible);
    handleSetMenu('open', open);
  };

  renderMark = props => {
    const { mark: { type } } = props;

    if (!marks[type]) {
      return null;
    }

    return marks[type](props);
  };

  renderNode = props => {
    const { node: { type } } = props;

    if (!nodes[type]) {
      return null;
    }

    return nodes[type](props);
  };

  // Render the editor.
  render() {
    const { menu, handleSetMenu, value } = this.props;
    if (!value) {
      return null;
    }

    return (
      <Fragment>
        <Toolbar value={value} onChange={this.onChange} />
        <div className="evt-editor__wrapper">
          <Editor
            value={value}
            onChange={this.onChange}
            renderNode={this.renderNode}
            renderMark={this.renderMark}
            renderPlaceholder={renderPlaceholder}
            schema={schema}
          />
          <Menu
            menu={menu}
            handleSetMenu={handleSetMenu}
            handleInsertGallery={this.insertGallery}
            handleInsertImage={this.insertImage}
          />
        </div>
        <Divider />
        <Timestamp datetime=" " />
      </Fragment>
    );
  }
}

export default StoryEditor;
