import DeleteButton from './delete';
import classnames from 'classnames';
import { layout } from '../../config/styles';

const { max, block } = layout;

const ImageNode = ({ attributes, children, editor, node }) => {
  const baseClassName = 'evt-image__figure';
  const id = node.data.get('id');
  const name = node.data.get('name');
  const isThumbnail = node.data.get('thumbnail');

  const className = classnames({
    [baseClassName]: true,
    [`${baseClassName}--thumbnail`]: isThumbnail
  });

  const src = `/assets/${id}/600/${name}`;

  if (!id) {
    return null;
  }

  return (
    <figure className={className} {...attributes}>
      <img className="evt-image" src={src} onClick={e => console.log(e)} />
      <DeleteButton
        isThumbnail={isThumbnail}
        title="Remove Image"
        editor={editor}
        node={node}
      />
      {children}
      <style jsx>{`
        .evt-image__figure {
          position: relative;
          margin: ${block * 2}px 0;
          width: 100%;
          max-width: ${max}px;
        }
        .evt-image__figure--thumbnail {
          margin: ${block}px 0 ${block}px ${block}px;
          max-width: 180px;
          display: inline-block;
        }
        .evt-image {
          width: 100%;
        }
      `}</style>
    </figure>
  );
};

export default ImageNode;
