import { Block } from 'slate';
import {
  CHILD_REQUIRED,
  CHILD_TYPE_INVALID,
  LAST_CHILD_TYPE_INVALID
} from 'slate-schema-violations';

const documentNormalizers = {
  [CHILD_TYPE_INVALID]: (change, violation, { child, index }) => {
    return change.setNodeByKey(child.key, index === 0 ? 'title' : 'paragraph');
  },
  [CHILD_REQUIRED]: (change, violation, { node, index }) => {
    const block = Block.create(index === 0 ? 'title' : 'paragraph');
    return change.insertNodeByKey(node.key, index, block);
  },
  [LAST_CHILD_TYPE_INVALID]: (change, violation, { node }) => {
    const paragraph = Block.create('paragraph');
    return change.insertNodeByKey(node.key, node.nodes.size, paragraph);
  }
};

const schema = {
  document: {
    last: { types: ['paragraph'] },
    nodes: [
      { types: ['title'], min: 1, max: 1 },
      { types: ['image', 'gallery', 'paragraph', 'headline', 'blockquote'] }
    ],
    normalize: (change, violation, options) => {
      const normalizer = documentNormalizers[violation];
      if (normalizer) {
        return documentNormalizers[violation](change, violation, options);
      }
    }
  },
  blocks: {
    paragraph: {
      nodes: [{ objects: ['text'] }]
    },
    gallery: {
      isVoid: false,
      nodes: [{ types: ['image'] }, { objects: ['text'] }]
    },
    image: {
      isVoid: false,
      data: { id: v => v }, // add validation check
      nodes: [{ objects: ['text'] }, { types: ['caption'], min: 1, max: 1 }],
      normalize: (change, violation, { node }) => {
        if (violation === CHILD_REQUIRED) {
          const block = Block.create('caption');
          return change.insertNodeByKey(node.key, 0, block);
        }
      }
    },
    caption: {
      nodes: [{ objects: ['text'] }]
    }
  }
};

export default schema;
