import IconButton from '../menu/button-icon';
import TextButton from '../menu/button-text';

const DEFAULT_NODE = 'paragraph';

const Toolbar = ({ value, onChange }) => {
  const onClickMark = (e, type) => {
    e.preventDefault();
    if (value.blocks.some(node => node.type === 'title')) {
      return;
    }

    const change = value.change().toggleMark(type);
    onChange(change);
  };

  const onClickBlock = (e, type) => {
    e.preventDefault();

    const hasBlock = value.blocks.some(node => node.type === type);
    const change = value.change().setBlocks(hasBlock ? DEFAULT_NODE : type);

    onChange(change);
  };

  return (
    <div className="toolbar mdc-typography">
      <TextButton onClick={e => onClickMark(e, 'bold')}>
        <strong>B</strong>
      </TextButton>
      <TextButton onClick={e => onClickMark(e, 'italic')}>
        <em>I</em>
      </TextButton>
      <TextButton onClick={e => onClickMark(e, 'underline')}>U</TextButton>
      <TextButton>A</TextButton>
      <TextButton onClick={e => onClickBlock(e, 'headline')}>H2</TextButton>
      <TextButton onClick={e => onClickBlock(e, 'blockquote')}>
        &rdquo;
      </TextButton>
      <IconButton>list</IconButton>
      <style jsx>{`
        .toolbar {
          padding-bottom: 4px;
          border-bottom: 1px solid hsla(0, 100%, 0%, 0.5);
          position: sticky;
          top: 64px;
          background: white;
          z-index: 10;
        }
        .toolbar :global(.evt-menu__action) {
          background: hsl(0, 100%, 100%);
          color: hsla(0, 100%, 0%, 0.5);
        }
      `}</style>
    </div>
  );
};

export default Toolbar;
