import { normalize } from 'normalizr';
import axios from 'axios';
import { Value } from 'slate';

// Router
import { Router } from '../../server/routes';

// Redux Actions
import { setCurrentEntity } from '@specialcircumstances/pkg-actual';
import { mergeEntities, setEntities, updateEntities } from '@specialcircumstances/pkg-entities';
import { setEditor } from '@specialcircumstances/pkg-editor';

// Schemas
import { storySchema, storyListSchema } from './schema';

// Selectors
import {
  storySelector,
  storyImagesSelector,
  storyTitleSelector
} from './selectors';

// API
import { postCommand } from './api';

import getInitialValue from './initial';

/**
 * @param { String } apiEndpoint
 */
export const fetchAllStories = apiEndpoint => {
  return async dispatch => {
    const { data: { data } } = await axios.get(`${apiEndpoint}/api/articles`);
    const { entities, result } = normalize(data, storyListSchema);

    dispatch(mergeEntities(entities));
    dispatch(setCurrentEntity('stories', result, 'success'));
  };
};

/**
 * @param { String } apiEndpoint
 * @param { String } id
 */
export const fetchStory = (apiEndpoint, id) => {
  return async dispatch => {
    const results = await axios.get(`${apiEndpoint}/api/articles/${id}`);
    console.log('results', results);
    const { data: { data } } = results;
    data.content = data.content
      ? Value.fromJSON(data.content)
      : Value.fromJSON(getInitialValue(data.title));
    const { entities, result } = normalize(data, storySchema);

    dispatch(mergeEntities(entities));
    dispatch(setCurrentEntity('story', result, 'success'));
  };
};

/**
 * @param { String } apiEndpoint
 * @param { String } id
 */
export const createStory = (apiEndpoint, payload) => {
  return async dispatch => {
    const { data } = await postCommand(apiEndpoint)('CreateStory', payload);
    const { id } = data;

    dispatch(updateEntities('stories', id, data));
    dispatch(setEditor('dirty', false));
    Router.pushRoute('story', { id });
  };
};

/**
 */
export const updateStory = (apiEndpoint, id, payload) => {
  return async (dispatch, getState) => {
    dispatch(setEntities('stories', id, 'content', payload));
    const state = getState();
    const isDirty = state.getIn(['editor', 'dirty']);

    // If the state isn't dirty, we don't make an API request
    if (!isDirty) {
      return;
    }

    // Move to single selector
    const story = storySelector(state);
    const images = storyImagesSelector(state);
    const title = storyTitleSelector(state);

    const { data, status } = await postCommand(apiEndpoint)('UpdateStory', {
      ...story,
      images,
      title
    });

    // const { entities } = normalize(data, storySchema);

    // dispatch(mergeEntities(entities));
    dispatch(setEditor('dirty', false));
    dispatch(setCurrentEntity('story', data._id, status));
  };
};
