import { createStore, compose, applyMiddleware } from 'redux';
import thunk from 'redux-thunk';
import { Map } from 'immutable';
import { composeWithDevTools } from 'redux-devtools-extension';

import getConfig from '@specialcircumstances/pkg-config';
import reducers from './reducers';

const dev = getConfig('DEV');
const browser = typeof window !== 'undefined';

/**
 * @param { Object } initialState - the initial state of the Redux store
 * @returns { Store } redux Store
 */
const configureStore = (initialState = {}) => {
  const middleware = [thunk];
  /* istanbul ignore next */
  const composeEnhancers = dev && browser ? composeWithDevTools : compose;

  const store = createStore(
    reducers,
    Map(initialState),
    composeEnhancers(applyMiddleware(...middleware))
  );

  return store;
};

export default configureStore;
