import test from 'ava';
import nock from 'nock';
import sinon from 'sinon';

import { fetchStory, fetchAllStories } from '../../../src/reducers';

const API_ENDPOINT = 'https://api.example.com';

const mockStory = {
  id: '5a5e7dc8e7bed',
  title: 'French Girls Know It All',
  slug: 'french-girls-know-it-all'
};

const requestMockStory = async id => {
  const dispatch = sinon.spy();
  const status = 200;

  nock(API_ENDPOINT)
    .get(`/api/articles/${id}`)
    .reply(status, {
      data: mockStory
    });

  await fetchStory(API_ENDPOINT, id)(dispatch);

  return dispatch;
};

test.skip('fetchStory dispatches the story entity', async t => {
  const { id } = mockStory;
  const dispatch = await requestMockStory(id);
  const { type, payload } = dispatch.args[0][0];

  t.is(type, 'ENTITIES_MERGE');
  t.deepEqual(payload, {
    stories: { [id]: mockStory }
  });
});

test('fetchStory dispatches the current story ID', async t => {
  const { id } = mockStory;
  const dispatch = await requestMockStory(id);
  const { type, payload } = dispatch.args[1][0];

  t.is(type, 'CURRENT_ENTITIES_SET');
  t.deepEqual(payload, {
    name: 'story',
    result: mockStory.id,
    status: 'success'
  });
});

const requestMockStories = async () => {
  const dispatch = sinon.spy();
  const status = 200;

  nock(API_ENDPOINT)
    .get(`/api/articles`)
    .reply(status, {
      data: [mockStory]
    });

  await fetchAllStories(API_ENDPOINT)(dispatch);

  return dispatch;
};

test('fetchAllStories should return an array of stories', async t => {
  const dispatch = await requestMockStories();
  const { type, payload } = dispatch.args[0][0];

  t.is(type, 'ENTITIES_MERGE');
  t.deepEqual(payload, {
    stories: { [mockStory.id]: mockStory }
  });
});

test('fetchAllStories dispatches an array of current story IDs', async t => {
  const { id } = mockStory;
  const dispatch = await requestMockStories(id);
  const { type, payload } = dispatch.args[1][0];

  t.is(type, 'CURRENT_ENTITIES_SET');
  t.deepEqual(payload, {
    name: 'stories',
    result: [mockStory.id],
    status: 'success'
  });
});
